
module Dieta

  # Clase que representa un menu dietético
  # @author alu0100896282
  # @attr [String] titulo Título del menu (Almuerzo, Cena, ...)
  # @attr [String] porcentaje Cadena de texto que representa la ingesta diara
  # @attr [Array[Array[String]]] platos Conjunto de platos con sus descripciones, porción recomendada e injesta
  # @attr [String] VCT Valor calórico total
  # @attr [String] proteinas Porcentaje de proteínas
  # @attr [String] grasas Porcentaje de grasas
  # @attr [String] hidratos Porcentaje de hidratos
  class Menu

    include Comparable

    attr_accessor :titulo, :porcentaje, :platos, :VCT, :proteinas, :grasas, :hidratos

    # Establece un orden de preferencia de acuerdo al VCT
    # @param [Menu] anOther Objeto de la clase Menu
    def <=>(anOther)

        @VCT.to_f <=> anOther.VCT.to_f

    end

    # Crea una instancia de la clase Menu
    # @param [Array[String]] cabecera Título y los porcentajes de la injesta diaria
    # @param [Array[Array[String]]] platos Conjunto de platos con descripción, porción e injesta recomendada
    # @param [Array[String]] informacion Conjunto con el VCT, porcentaje de proteinas, grasas e hidratos
    # @return [Menu] Nueva instancia de la clase Menu
    def initialize(cabecera, platos, informacion)

        @titulo       = cabecera[0]

        @porcentaje   = cabecera[1].to_s
        if cabecera[2]
            @porcentaje  += "-" + cabecera[2].to_s
        end
        @porcentaje  += "%"

        @platos       = platos
        @VCT          = informacion[0]
        @proteinas    = informacion[1]
        @grasas       = informacion[2]
        @hidratos     = informacion[3]

    end

    # Devuelve información sobre un plato
    # @param [Number] numero Número de plato
    # @return [Array[String]] Información del plato
    def get_plato(numero)
        @platos[numero]
    end

    # Devuelve solo la descripción de un plato
    # @param [Number] numero Número de plato
    # @return [String] Descripción del plato
    def get_descripcion(numero)
        @platos[numero][0]
    end

    # Devuelve solo la porción recomendada de un plato
    # @param [Number] numero Número de plato
    # @return [String] Porción recomendada del plato
    def get_porcion(numero)
        @platos[numero][1]
    end

    # Devuelve solo la ingesta recomendada de un plato
    # @param [Number] numero Número de plato
    # @return [String] Injesta recomendada del plato
    def get_ingesta(numero)
        @platos[numero][2]
    end

    # Transforma en una cadena
    # @return [String] Cadena con información formateada del Menu
    def to_s

        mostrar = "#{@titulo} (#{@porcentaje})\n"

        i = 0;
        while i < @platos.length
            mostrar += "- #{@platos[i][0]}, #{platos[i][1]}, #{platos[i][2]}\n"
            i+=1
        end

        mostrar += "V.C.T. | % #{@VCT} kcal | #{@proteinas}% - #{@grasas}% - #{@hidratos}%\n"

    end

 end
end

# Clase hija de menu dietéticos orientados a un grupo de alimentos
# @author alu0100896282
# @attr [String] grupo Nombre del grupo de alimentos
class GruposAlimentos < Dieta::Menu

    attr_accessor :grupo

    # Crea una instancia de la clase Menu
    # @param [String] grupo Nombre del grupo de alimentos
    # @param [Array[String]] cabecera Título y los porcentajes de la injesta diaria
    # @param [Array[Array[String]]] platos Conjunto de platos con descripción, porción e injesta recomendada
    # @param [Array[String]] informacion Conjunto con el VCT, porcentaje de proteinas, grasas e hidratos
    # @return [Menu] Nueva instancia de la clase Menu
    def initialize(grupo, cabecera, platos, informacion)
        super(cabecera, platos, informacion)
        @grupo = grupo
    end

    # Transforma en una cadena
    # @return [String] Cadena con información formateada del Menu
    def to_s
        "#{@grupo}\n#{super.to_s}"
    end

end

# Clase hija de menu dietéticos orientados a un grupo de edad
# @author alu0100896282
# @attr [String] edad Nombre del grupo de edad
class GruposEdad < Dieta::Menu

    attr_accessor :edad

    # Crea una instancia de la clase Menu
    # @param [String] edad Nombre del grupo de edad
    # @param [Array[String]] cabecera Título y los porcentajes de la injesta diaria
    # @param [Array[Array[String]]] platos Conjunto de platos con descripción, porción e injesta recomendada
    # @param [Array[String]] informacion Conjunto con el VCT, porcentaje de proteinas, grasas e hidratos
    # @return [Menu] Nueva instancia de la clase Menu
    def initialize(edad, cabecera, platos, informacion)
        super(cabecera, platos, informacion)
        @edad = edad
    end

    # Transforma en una cadena
    # @return [String] Cadena con información formateada del Menu
    def to_s
        "#{@edad}\n#{super.to_s}"
    end

end


class Menu_DSL < Dieta::Menu

    def initialize(tag, &block)
        super([], [], [])

        if block_given?
            if block.arity == 1
                yield self
            else
                instance_eval(&block)
            end
        end
    end

    def titulo(name)
        @titulo = name
    end

    def ingesta(options = {})
        @porcentaje = "#{options[:min]}"
        if options[:max]
            @porcentaje  += "-#{options[:max]}"
        end
        @porcentaje  += "%"
    end

    def plato(options = {})
        plato = [];
        plato << options[:descripcion]
        plato << options[:porcion]
        plato << options[:gramos].to_s

        @platos << plato
    end

    def porcentajes(options = {})
        @VCT = options[:vct].to_s
        @proteinas = options[:proteinas].to_s
        @grasas = options[:grasas].to_s
        @hidratos = options[:hidratos].to_s
    end
end
